#ifndef KALMAN_FILTER_HPP
#define KALMAN_FILTER_HPP
#include <sta/math/linalg/matrix.hpp>

/**
 * @file kalmanFilter.hpp
 * @brief Contains the definition of the KalmanFilter class and related structures.
 */

namespace sta
{

namespace math
{

/**
 * @struct KalmanState
 * @brief Represents the state of the Kalman filter.
 */
struct KalmanState
{
	matrix error; ///< The error covariance matrix.
	matrix x;     ///< The state vector.
};

/**
 * @class KalmanFilter
 * @brief Represents a Kalman filter.
 */
class KalmanFilter
{
public:
	/**
	 * @brief Constructs a KalmanFilter object.
	 * @param F The state transition matrix.
	 * @param B The control input matrix.
	 * @param H The observation matrix.
	 * @param Q The process noise covariance matrix.
	 * @param R The measurement noise covariance matrix.
	 */
	KalmanFilter(matrix F, matrix B, matrix H, matrix Q, matrix R);

	/**
	 * @brief Destroys the KalmanFilter object.
	 */
	~KalmanFilter();

	/**
	 * @brief Predicts the next state of the Kalman filter, based on the current state and a control input.
	 * @param state The current state and uncertainty convariance matrix.
	 * @param u The control input.
	 * @return The predicted state of the Kalman filter.
	 */
	KalmanState predict(KalmanState state, matrix u);

	/**
	 * @brief Predicts the next state of the Kalman filter, based on the current state.
	 * 
	 * @param state The current state and uncertainty convariance matrix.
	 * @return The predicted state of the Kalman filter. 
	 */
	KalmanState predict(KalmanState state);

	/**
	 * @brief Corrects the state of the Kalman filter based on a measurement.
	 * @param state The current state and error convariance matrix.
	 * @param z The observed measurement.
	 * @return The corrected state of the Kalman filter.
	 */
	KalmanState correct(KalmanState state, matrix z);

	/**
	 * @brief Corrects the state of the Kalman filter based on a measurement. The observation matrix and measurement noise covariance matrix are passed as arguments.
	 * @param state The current state and error convariance matrix.
	 * @param z The observed measurement.
	 * @param H The observation matrix.
	 * @param R The measurement noise covariance matrix.
	 * @return The corrected state of the Kalman filter.
	 */
	KalmanState correct(KalmanState state, matrix z, matrix H, matrix R);
private:
	matrix F_;         ///< The state transition matrix.
	matrix B_;         ///< The control input matrix.
	matrix H_;         ///< The observation matrix.
	matrix Q_;         ///< The process noise covariance matrix.
	matrix R_;         ///< The measurement noise covariance matrix.
	uint8_t n_;        ///< The dimension of the state vector.
	matrix identity_;  ///< The identity matrix with size of the state vector.
};
    
} // namespace math

} // namespace sta
#endif // KALMAN_FILTER_HPP
